@@ -2,7 +2,7 @@
  * configuration.c: logapp configuration handling
  *
  *
- * Copyright (C) 2007 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2008 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@ config_t config = {
 	.dumbterm		= 0,
 	.detectescape		= 0,
 	.usepty			= 1,
+	.ptyremovecr		= 1,
 	.appendlog		= 0,
 	.printsummary		= 0,
 	.printlogname		= 0,
@@ -51,17 +52,20 @@ config_t config = {
 	.logname		= "./logapp.log",
 	.logtime		= 0,
 	.logreltime		= 0,
+	.logenv			= 0,
 	.circularlog		= 0,
 	.maxlogsize		= 0,
 	.locklogfile		= 1,
 	.warnlogfilelock	= 1,
 	.maxlogfiles		= 10,
 	.logrename		= 1,
-	.alignlog               = 1,
+	.alignlog		= 1,
 	.jointimeout		= 0,
 	.configfile		= NULL,
 	.configsection		= NULL,
 	.custconfigfile		= NULL,
+	.preexeccmd		= NULL,
+	.postexeccmd		= NULL,
 };
 
 /* default search path list for configuration file */
@@ -79,7 +83,13 @@ pipe_t pstdout = {
 	.name		= "stdout",
 	.buf		= NULL,
 	.charbased	= 0,
-	.blen		= 2048,
+	.detectescape	= 0,
+	.switchtocbmode = 0,
+	.blen		= 65540,
+	.bfill		= 0,
+	.rblen		= 4096,
+	.memcr		= 0,
+	.recalclen	= 0,
 	.fgcol		= -1,
 	.bgcol		= -1,
 	.bold		= 0,
@@ -88,10 +98,12 @@ pipe_t pstdout = {
 	.escreset	= "\033[0m",
 	.esccolor	= NULL,
 	.linecount	= 0,
-	.lineprefix     = NULL,
-	.regexp         = NULL,
+	.lineprefix	= NULL,
+	.regexp		= NULL,
 	.regbgcol	= 4,
-	.bgesccolor    = NULL,
+	.bgesccolor	= NULL,
+	.execregexp	= NULL,
+	.execcommand	= NULL,
 };
 
 /* configuration for stderr */
@@ -101,7 +113,13 @@ pipe_t pstderr = {
 	.name		= "stderr",
 	.buf		= NULL,
 	.charbased	= 0,
-	.blen		= 2048,
+	.detectescape	= 0,
+	.switchtocbmode = 0,
+	.blen		= 65540,
+	.bfill		= 0,
+	.rblen		= 4096,
+	.memcr		= 0,
+	.recalclen	= 0,
 	.fgcol		= 1,
 	.bgcol		= -1,
 	.bold		= 1,
@@ -111,22 +129,24 @@ pipe_t pstderr = {
 	.esccolor	= NULL,
 	.linecount	= 0,
 	.lineprefix	= "STDERR: ",
-	.regexp         = NULL,
+	.regexp		= NULL,
 	.regbgcol	= 4,
-	.bgesccolor    = NULL,
+	.bgesccolor	= NULL,
+	.execregexp	= NULL,
+	.execcommand	= NULL,
 };
 
 /* logfile configuration */
 logfile_t logfile = {
-	.fh	= 0,
-	.name	= NULL,
-	.sizelimit = 0,
-	.oldext = ".old",
-	.appendnr = 0,
-	.head	= ">> logapp:",
-	.split	= "== logapp split:",
-	.foot	= "<< logapp end:",
-	.addnewline = 0,
+	.fh		= 0,
+	.name		= NULL,
+	.sizelimit	= 0,
+	.oldext		= ".old",
+	.appendnr	= 0,
+	.head		= ">> logapp:",
+	.split		= "== logapp split:",
+	.foot		= "<< logapp end:",
+	.addnewline	= 0,
 };
 
 struct winsize ptysize;
@@ -140,7 +160,7 @@ app_t app = {
 	.pid		= 0,
 	.pstdout	= &pstdout,
 	.pstderr	= &pstderr,
-	.ptysize        = &ptysize,
+	.ptysize	= &ptysize,
 	.ptytermios	= &ptytermios,
 	.ptytermios_bak = &ptytermios_bak,
 	.active		= 0,
@@ -180,6 +200,9 @@ const stringvalue_t clipvalues[] = {
 
 /* all possible configuration parameters with help message */
 arglist_t arglist[] = {
+	/* General options */
+	{ '\0', NULL,		NULL,		TNONE,	NULL,
+		"General options:", 0 },
 	{ '?',	"help",		NULL,		TNONE,	&show_usage,
 		"show this help", 0 },
 	{ '\0', "configfile",   "FILE",		TSTRING,
@@ -194,81 +217,111 @@ arglist_t arglist[] = {
 		&config.disable_keywords, "comma separated keywords to "
 		"disable output handling when found in argument list", 0 },
 	{ '\0', "detectescape",	"BOOL",		TBOOL,	&config.detectescape,
-		"switch to charbased mode if escape sequence is detected", 0},
+		"switch to charbased mode if escape sequence is detected", 0 },
+#if CONFIG_SUPPORT_PTY
+	{ '\0', "usepty",	"BOOL",		TBOOL,	&config.usepty,
+		"use PTY for stream redirection", 0 },
+	{ '\0', "ptyremovecr",  "BOOL",		TBOOL,  &config.ptyremovecr,
+		"translate CR-LF to LF when capturing PTY", 0 },
+#else
+	{ '\0', "usepty",	"BOOL",		TBOOL,	&config.usepty,
+		"<not supported> use PTY for stream redirection", 0 },
+	{ '\0', "ptyremovecr",  "BOOL",		TBOOL,  &config.ptyremovecr,
+		"<not supported> translate CR-LF to LF when capturing PTY",
+		0 },
+#endif
 	{ '\0', "stdout_blen",	"SIZE",		TUINT,	&pstdout.blen,
 		"stdout buffer size", 0 },
 	{ '\0', "stderr_blen",	"SIZE",		TUINT,	&pstderr.blen,
 		"stderr buffer size", 0 },
-	{ '\0', "dumbterm",	"BOOL",		TBOOL,	&config.dumbterm,
-		"disable colors for use with dumb terminals", 0 },
-#if CONFIG_SUPPORT_PTY
-	{ '\0', "usepty",	"BOOL",		TBOOL,	&config.usepty,
-		"use PTY for stream redirection", 0 },
-#else
-	{ '\0', "usepty",	"BOOL",		TBOOL,	&config.usepty,
-		"<not supported> use PTY for stream redirection", 0 },
-#endif
+	{ '\0', "stdout_charbased",	"BOOL",	TBOOL,  &pstdout.charbased,
+		"handle stdout char- instead of line-based", 0 },
+	{ '\0', "stderr_charbased",	"BOOL",	TBOOL,  &pstderr.charbased,
+		"handle stderr char- instead of line-based", 0 },
+	/* Logging options */
+	{ '\0', NULL,		NULL,		TNONE,	NULL,
+		"Logging options:", 0 },
 	{ 'l',  "logfile",	"NAME",		TSTRING, &config.logname,
 		"application logfile", 0 },
 	{ 'a',  "appendlog",	"BOOL",		TBOOL, &config.appendlog,
 		"append to existing logfile", 0 },
-	{ 't',  "logtime",	"BOOL",	TBOOL, &config.logtime,
-		"add timestamp to each logged line", 0 },
-	{ '\0',  "logreltime",	"BOOL",	TBOOL, &config.logreltime,
-		"log relative time with --logtime", 0 },
-	{ '\0', "alignlog",	"BOOL",	TBOOL, &config.alignlog,
-		"use one line for every write in charbased mode", 0},
-	{ '\0', "jointimeout",	"TIME",	TUINT, &config.jointimeout,
-		"join timeout for packets with alignlog active", 0},
-	{ '\0', "locklogfile",	"BOOL",	TBOOL, &config.locklogfile,
-		"lock logfile when opening it", 0},
-	{ '\0', "warnlogfilelock", "BOOL",	TBOOL, &config.warnlogfilelock,
-		"print a warning if current logfile is locked", 0},
-	{ '\0', "maxaltlogfiles", "COUNT",	TUINT, &config.maxlogfiles,
-		"max # of alternate logfiles on lock", 0},
 	{ '\0', "maxlogsize",	"SIZE",	TUINT, &config.maxlogsize,
 		"max. logfile size in KiB (0=no limit, 10-4000000) the file "
 		"will be truncated if logrename isn't set", 0 },
 	{ '\0', "logrename",	"BOOL",	TBOOL, &config.logrename,
-		"rename logfile before replacing it", 0},
+		"rename logfile before replacing it", 0 },
 	{ '\0', "circularlog",	"BOOL",	TBOOL, &config.circularlog,
-		"write log in a circular way, keeping the max. size", 0},
+		"write log in a circular way, keeping the max. size", 0 },
 	{ '\0', "oldlogext",	"STRING",	TSTRING, &logfile.oldext,
-		"extension for old logfile", 0},
-	{ 's',  "print_summary", "BOOL",	TBOOL, &config.printsummary,
-		"print execution summary", 0 },
+		"extension for old logfile", 0 },
+	{ '\0', "locklogfile",	"BOOL",	TBOOL, &config.locklogfile,
+		"lock logfile when opening it", 0 },
+	{ '\0', "warnlogfilelock", "BOOL",	TBOOL, &config.warnlogfilelock,
+		"print a warning if current logfile is locked", 0 },
 	{ 'n',  "print_logname", "BOOL",	TBOOL, &config.printlogname,
 		"print logfile name after execution", 0 },
-	{ 'f',  "stdout_fgcol",	"COLOR#",	TCOLOR, &pstdout.fgcol,
+	{ '\0', "maxaltlogfiles", "COUNT",	TUINT, &config.maxlogfiles,
+		"max # of alternate logfiles on lock", 0 },
+	{ '\0', "alignlog",	"BOOL",	TBOOL, &config.alignlog,
+		"use one line for every write in charbased mode", 0 },
+	{ '\0', "jointimeout",	"TIME",	TUINT, &config.jointimeout,
+		"join timeout for packets with alignlog active", 0 },
+	{ 't',  "logtime",	"BOOL",	TBOOL, &config.logtime,
+		"add timestamp to each logged line", 0 },
+	{ '\0',  "logreltime",	"BOOL",	TBOOL, &config.logreltime,
+		"log relative time with --logtime", 0 },
+	{ '\0',  "logenv",	"BOOL",	TBOOL, &config.logenv,
+		"add environment variables to logfile", 0 },
+	{ 'p',  "stdout_lineprefix",   "STRING",	TSTRING,
+		&pstdout.lineprefix, "logfile line prefix for stdout", 0 },
+	{ 'P',  "stderr_lineprefix",   "STRING",	TSTRING,
+		&pstderr.lineprefix, "logfile line prefix for stderr", 0 },
+	/* Console output options */
+	{ '\0', NULL,		NULL,		TNONE,	NULL,
+		"Console output options:", 0 },
+	{ '\0', "dumbterm",	"BOOL",		TBOOL,	&config.dumbterm,
+		"disable colors for use with dumb terminals", 0 },
+	{ 's',  "print_summary", "BOOL",	TBOOL, &config.printsummary,
+		"print execution summary", 0 },
+	{ 'f',  "stdout_fgcol",	"COLOR",	TCOLOR, &pstdout.fgcol,
 		"stdout console foreground color (disable=-1, 0-7)", 0 },
-	{ 'F',  "stderr_fgcol",	"COLOR#",	TCOLOR,   &pstderr.fgcol,
+	{ 'F',  "stderr_fgcol",	"COLOR",	TCOLOR,   &pstderr.fgcol,
 		"stderr console foreground color (disable=-1, 0-7)", 0 },
 	{ 'b',  "stdout_bold",	"BOOL",		TBOOL,	&pstdout.bold,
 		"bold stdout console font", 0 },
 	{ 'B',  "stderr_bold",	"BOOL",		TBOOL,	&pstderr.bold,
 		"bold stderr console font", 0 },
-	{ '\0', "stdout_regexp_bgcol",	"COLOR#", TCOLOR, &pstdout.regbgcol,
-		"stdout console background color on regexp match (disable=-1, "
-		"0-7)", 0 },
-	{ '\0', "stderr_regexp_bgcol",	"COLOR#", TCOLOR, &pstderr.regbgcol,
-		"stderr console background color on regexp match (disable=-1, "
-		"0-7)", 0 },
-	{ 'c',  "stdout_clip",	"LENGTH",	TCLIP,	&pstdout.clip,
-		"clip stdout console at column LENGTH (disable=-1, auto=-2)", 0 },
-	{ 'C',  "stderr_clip",	"LENGTH",	TCLIP,	&pstderr.clip,
-		"clip stderr console at column LENGTH (disable=-1, auto=-2)", 0 },
-	{ 'p',  "stdout_lineprefix",   "STRING",	TSTRING,
-		&pstdout.lineprefix, "logfile line prefix for stdout", 0 },
-	{ 'P',  "stderr_lineprefix",   "STRING",	TSTRING,
-		&pstderr.lineprefix, "logfile line prefix for stderr", 0 },
 	{ 'r',  "stdout_regexp", "STRING", TSTRING, &pstdout.regexp,
 		"regular expression to change stdout background color", 0 },
 	{ 'R',	"stderr_regexp",   "STRING",	TSTRING, &pstderr.regexp,
 		"regular expression to change stderr background color", 0 },
-	{ '\0', "stdout_charbased",	"BOOL",	TBOOL,  &pstdout.charbased,
-		"handle stdout char- instead of line-based", 0 },
-	{ '\0', "stderr_charbased",	"BOOL",	TBOOL,  &pstderr.charbased,
-		"handle stderr char- instead of line-based", 0 },
+	{ '\0', "stdout_regexp_bgcol",	"COLOR", TCOLOR, &pstdout.regbgcol,
+		"stdout console background color on regexp match (disable=-1, "
+		"0-7)", 0 },
+	{ '\0', "stderr_regexp_bgcol",	"COLOR", TCOLOR, &pstderr.regbgcol,
+		"stderr console background color on regexp match (disable=-1, "
+		"0-7)", 0 },
+	{ 'c',  "stdout_clip",	"LENGTH",	TCLIP,	&pstdout.clip,
+		"clip stdout console at column LENGTH (disable=-1, auto=-2)",
+		0 },
+	{ 'C',  "stderr_clip",	"LENGTH",	TCLIP,	&pstderr.clip,
+		"clip stderr console at column LENGTH (disable=-1, auto=-2)",
+		0 },
+	/* Command execution options */
+	{ '\0', NULL,		NULL,		TNONE,	NULL,
+		"Command execution options:", 0 },
+	{ '\0', "preexec",	"STRING",	TSTRING, &config.preexeccmd,
+		"command executed before application start", 0 },
+	{ '\0', "postexec",	"STRING",	TSTRING, &config.postexeccmd,
+		"command executed after application exit", 0 },
+	{ 'e',  "stdout_execregexp", "STRING", TSTRING, &pstdout.execregexp,
+		"regular expression to execute command", 0 },
+	{ 'E',	"stderr_execregexp",   "STRING", TSTRING, &pstderr.execregexp,
+		"regular expression to execute command", 0 },
+	{ 'x',	"stdout_execcommand", "STRING", TSTRING, &pstdout.execcommand,
+		"command to be executed after stdout regexp match", 0 },
+	{ 'X',	"stderr_execregexp",   "STRING", TSTRING, &pstderr.execcommand,
+		"command to be executed after stderr regexp match", 0 },
 };
 
 const int arglistsize = ((int)(sizeof(arglist)/sizeof(arglist_t)));
@@ -295,6 +348,7 @@ void show_configuration(void)
 	message("  append to logfile       %i\n", config.appendlog);
 	message("  log timestamps          %i\n", config.logtime);
 	message("  log relative time       %i\n", config.logreltime);
+	message("  log environment         %i\n", config.logenv);
 	message("  align log writes left   %i\n", config.alignlog);
 	message("  join timeout            %i\n", config.jointimeout);
 	message("  lock logfiles           %i\n", config.locklogfile);
@@ -313,6 +367,10 @@ void show_configuration(void)
 			app.argv[i]);
 	}
 
+	message("\n");
+	message("  preexec command         %s\n", config.preexeccmd);
+	message("  postexec command        %s\n", config.postexeccmd);
+
 	message("\n");
 	if (CONFIG_USE_THREADS) {
 		message("  use threads             %i\n", 1);
@@ -321,8 +379,10 @@ void show_configuration(void)
 	}
 	if (CONFIG_SUPPORT_PTY) {
 		message("  use PTY                 %i\n", config.usepty);
+		message("  remove CR for PTY       %i\n", config.ptyremovecr);
 	} else {
 		message("  use PTY                 <not supported>\n");
+		message("  remove CR for PTY       <not supported>\n");
 	}
 
 	message("\n  dumb terminal mode      %i\n", config.dumbterm);
@@ -331,7 +391,6 @@ void show_configuration(void)
 	message("  disable output handling %i\n", config.disable);
 	message("  disable keywords        %s\n",
 		config.disable_keywords?config.disable_keywords:"<none>");
-	message("  esc-sequence detection  %i\n", config.detectescape);
 
 	message("\n  stdout buffer length    %i\n", app.pstdout->blen);
 	message("  stdout foreground color %i\n", app.pstdout->fgcol);
@@ -350,7 +409,12 @@ void show_configuration(void)
 		app.pstdout->lineprefix:"<none>");
 	message("  stdout regexp           %s\n", app.pstdout->regexp?
 		app.pstdout->regexp:"<disabled>");
+	message("  stdout exec regexp      %s\n", app.pstdout->execregexp?
+		app.pstdout->execregexp:"<disabled>");
+	message("  stdout regexp command   %s\n", app.pstdout->execcommand?
+		app.pstdout->execcommand:"<none>");
 	message("  stdout charbased        %i\n", app.pstdout->charbased);
+	message("  stdout esc detection    %i\n", app.pstdout->detectescape);
 
 	message("\n  stderr buffer length    %i\n", app.pstderr->blen);
 	message("  stderr foreground color %i\n", app.pstderr->fgcol);
@@ -369,7 +433,12 @@ void show_configuration(void)
 		app.pstderr->lineprefix:"<none>");
 	message("  stderr regexp           %s\n", app.pstderr->regexp?
 		app.pstderr->regexp:"<disabled>");
+	message("  stderr exec regexp      %s\n", app.pstderr->execregexp?
+		app.pstderr->execregexp:"<disabled>");
+	message("  stderr regexp command   %s\n", app.pstderr->execcommand?
+		app.pstderr->execcommand:"<none>");
 	message("  stderr charbased        %i\n", app.pstderr->charbased);
+	message("  stderr esc detection    %i\n", app.pstderr->detectescape);
 }
 
 char* get_longpath(const char* filename)
@@ -403,6 +472,9 @@ char* get_longpath(const char* filename)
 
 static int fixup_pipe(pipe_t *pipe)
 {
+	if (config.detectescape)
+		pipe->detectescape = 1;
+
 	if ((pipe->fgcol < -1) || (pipe->fgcol > 7)) {
 		error("%s foreground color out of range\n", pipe->name);
 		return -1;
@@ -491,6 +563,27 @@ static int fixup_pipe(pipe_t *pipe)
 		}
 	}
 
+	if (pipe->execregexp) {
+		if (strlen(pipe->execregexp)) {
+			if (regcomp(&pipe->pexecreg, pipe->execregexp,
+				    REG_NOSUB)) {
+				error("unable to process regular expression "
+				      "for %s\n", pipe->name);
+				return -1;
+			}
+		} else {
+			pipe->execregexp = NULL;
+		}
+		/* disable execregexp if there was no command provided */
+		if (pipe->execcommand == NULL) {
+			pipe->execregexp = NULL;
+		} else {
+			if (strlen(pipe->execcommand) == 0) {
+			pipe->execregexp = NULL;
+			}
+		}
+	}
+
 	return 0;
 }
 
